--!parent
--!encoding=utf-8

--****************************************************************************
-- @package   Qt OOP API
-- @version  	0.3
-- @requires  Xtra("QtWidgets")
-- @author   	Valentin Schmidt
--****************************************************************************

global $

property ancestor
property _id
property _class

----------------------------------------
-- @constructor
----------------------------------------
on new (me)

  -- libs
  $.import("regex")
  
  me._class = #Translator
  me.ancestor = $.include($.Qt.libDir&"classes/iObject.ls").new()
  me._id = $.Qt._qx.newObject(me._class)
  return me
end

-- static
on _make (scr, id)
	obj = scr.rawNew()
	obj.ancestor = $.include($.Qt.libDir&"classes/iObject.ls").new()
	obj._class = #Translator
	obj._id = id
	return obj
end

----------------------------------------
-- ACTIONS
----------------------------------------

-- objectName () -> string
on objectName (me)
  return $.Qt._qx.callObject(me._id,#objectName)
end

-- setObjectName (string)
on setObjectName (me,p1)
  $.Qt._qx.callObject(me._id,#setObjectName,p1)
end

-- load (string,*string)
on load (me,p1,p2)
	if voidP(p2) then
  	return $.Qt._qx.callObject(me._id,#load,p1)
  else
  	return $.Qt._qx.callObject(me._id,#load,p1,p2)
  end if
end

-- translate (string,string) -> string
on translate (me,p1,p2)
  return $.Qt._qx.callObject(me._id,#translate,p1,p2)
end

----------------------------------------
-- custom actions
----------------------------------------

----------------------------------------
-- Activates/Deactivates translation of include scripts via script preprocessing
-- @param {bool}
----------------------------------------
on setIncludeTranslationEnabled (me, flag)
	if flag then
  	$.installPreprocessorInclude(#slotProcessIncludes, me)
  else
  	$.uninstallPreprocessorInclude()
  end if
end

----------------------------------------
-- Translates UI XML string
-- @param {string} ui
-- @param {string} context - e.g. filename of UI file
-- @return {string}
----------------------------------------
on trUI (me, ui, context)
  return $.regex.replace_callback_index("<string>([^<]*)</string>",ui,1,#slotUiReplace,[me],"g", context)
end

----------------------------------------
-- Translates StandardButtons e.g. in DialogButtonBox
-- @param {string} w - parentWidget
-- @param {string} context
----------------------------------------
on trStandardButtons (me, w, context)
  if voidP(context) then context = "StandardButtons"
  btns = w.findChildren(0, #PushButton)
  repeat with btn in btns
    str = btn.text()
    res = me.translate(context, str)
    if res<>"" then btn.setText(res)
    --else put str
  end repeat
end

----------------------------------------
-- @callback
----------------------------------------
on slotProcessIncludes (me, code, basename)
  return $.regex.replace_callback_index("\(\("&QUOTE&"([^"&QUOTE&"]*)"&QUOTE&"\)\)",code,1,#slotIncludeReplace,[me],"g", basename)
end

----------------------------------------
-- @callback
----------------------------------------
on slotIncludeReplace (me, str, basename)
  res = me.translate(basename, str)
  if res="" then
    --put basename && str
    res = str
  end if
  return QUOTE&res&QUOTE
end

----------------------------------------
-- @callback
----------------------------------------
on slotUiReplace (me, str, context)
  res = me.translate(context, str)
  --if res="" then res = me.translate("app", str) -- TODO
  if res="" then
    --put basename && str
    res = str
  end if
  return "<string>"&res&"</string>"
end

----------------------------------------
-- STATIC
----------------------------------------

----------------------------------------
-- SIGNALS
----------------------------------------
